/* backlight.c
 *
 * turns ALI LCD screen backlight on and off.
 * last modified 03/27/00
 *
 * Copyright (C) 2000  Transmeta Corporation
 *
 * written by Claudio Fleiner <cfl@transmeta.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "busybox.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>

#if defined BB_BACKLIGHT
const char backlight_usage[] =
        "backlight [0|1|on|off|flip]\n"
#ifndef BB_FEATURE_TRIVIAL_HELP
        "\nTurn the light of an SMI LCD screen on and off\n"
#endif
        ;
#endif 

int backlight_main(int argc, char **argv)
{
        unsigned int on,off,pdr22,pdr34;

        if (argc != 2) usage(backlight_usage);

        if (ioperm(0x3c4, 2, 1)) {
                fprintf(stderr,"ioperm() failed.  Please run as root.\n");
                exit(1);
        }

        on=0;
        off=0;

        outb(0x22,0x3c4);
        pdr22=inb(0x3c5);
        outb(0x34,0x3c4);
        pdr34=inb(0x3c5);

        on=!strcmp(argv[1],"on") || !strcmp(argv[1],"1");
        off=!strcmp(argv[1],"off") || !strcmp(argv[1],"0");
        if(!on && !off && !strcmp(argv[1],"flip")) {
                if(pdr22&0x8) off=1;
                else on=1;
        }

        if(on) {
                pdr22|=0xf;
                pdr34|=0x80;
        } else {
                pdr22&=0xf0;
                pdr34&=0x7f;
        }
        outb(0x22,0x3c4);
        outb(pdr22,0x3c5);
        outb(0x34,0x3c4);
        outb(pdr34,0x3c5);
        exit(0);
} 
