/*
 * keydown - is a key down? (written by Daniel Quinlan)
 *
 * Copyright (C) 2000 Transmeta Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "busybox.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <fcntl.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <sys/time.h>

/* START code from "Unix Programming Frequently Asked Questions" */
static struct termios stored_settings;

static void set_keypress(void)
{
	struct termios new_settings;

	tcgetattr(0, &stored_settings);

	new_settings = stored_settings;

	/* Disable canonical mode, and set buffer size to 1 byte */
	new_settings.c_lflag &= (~ICANON);
	new_settings.c_cc[VTIME] = 0;
	new_settings.c_cc[VMIN] = 1;

	tcsetattr(0, TCSANOW, &new_settings);
	return;
}

static void reset_keypress(void)
{
	tcsetattr(0, TCSANOW, &stored_settings);
	return;
}
/* END code from "Unix Programming Frequently Asked Questions" */

static void signal_handler ()
{
	reset_keypress();
	exit(1);
}

#if 0
void error_warn (char *str)
{
	fprintf(stderr, "%s: %s: %s\n", progname, str, strerror(errno));
}

void error_die (char *str)
{
	fprintf(stderr, "%s: %s: %s\n", progname, str, strerror(errno));
	exit(1);
}
#endif

#if defined BB_KEYDOWN
const char keydown_usage[] =
        "keydown key\n"
#ifndef BB_FEATURE_TRIVIAL_HELP
        "\nReturn true if the user is pressing down the given key.\n"
#endif
        ;
#endif 

int keydown_main(int argc, char **argv)
{
	struct itimerval itimer;
	int input = 0;
	char key = 0;

	/* command line options */
	if (argc != 2 || strlen(argv[1]) != 1) {
                usage(keydown_usage);
	}
	key = argv[1][0];	

	/* check for a keypress */
	itimer.it_interval.tv_sec = 0;
	itimer.it_interval.tv_usec = 0;
	itimer.it_value.tv_sec = 0;
	itimer.it_value.tv_usec = 50000;	/* 0.05s should be enough */
	set_keypress();
	signal(SIGINT, signal_handler);
	signal(SIGTSTP, signal_handler);
	signal(SIGALRM, signal_handler);
	signal(SIGTERM, signal_handler);
	setitimer(ITIMER_REAL, &itimer, NULL);
	input = getc(stdin);
	signal(SIGALRM, SIG_IGN);
	if ((char) input == key) {
		reset_keypress();
		exit(0);
	}
	reset_keypress();
	exit(1);
}
