/*
 * Copyright (C) 2000  Transmeta Corporation
 *
 * written by Claudio Fleiner <cfl@transmeta.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>   
#include <stdio.h>   
#include <sys/time.h>
#include <time.h>
#include <mlconfig.h>

#define TIMEZONE "/timezone"    // timezone file
#define ZONEDIR "/usr/share/zoneinfo"      // directory where timezone as stored

int selectname(const struct dirent *dirent) {
        return dirent->d_name[0]>='A' && dirent->d_name[0]<='Z';
}

void main(int argc,char *argv[]) {
        FILE *f;
        int i;
        int sc;
        char current[200];
        char tzfile[300];
        char *config;
        char *tz=NULL;
        struct timeval now;
        struct tm *tm;
        char *button;
        char *auth;

        config=getenv("CONFIG_WRITE");
        if(config==NULL) 
                HTML_ERROR("environment variable CONFIG_WRITE not defined");
        if(strlen(config)>280)
                HTML_ERROR("environment variable CONFIG_WRITE too long");
        strcpy(tzfile,config);
        strcat(tzfile,TIMEZONE);
        f=fopen(tzfile,"r");
        if(f==NULL) current[0]=0;
        else {
                fgets(current,200,f);
                current[strlen(current)-1]=0;
                fclose(f);
        }

        tz=(char*)getenv("Q_TZ");
        button=(char*)getenv("S_BUTTON");
        auth=(char*)getenv("AUTHORIZATION");


        if(button!=NULL && !strcmp(button,"Set Timezone") && tz!=NULL) {
                // set timezone
                int i;
                char path[300];
                struct stat sb;

                if(!AUTHORIZATION_ROOT) HTML_AUTHORIZATION_REQUIRED;

                strcpy(path,ZONEDIR "/");
                for(sc=i=0;tz[i];i++) {
                        switch(tz[i]) {
                        case '/': sc++;break;
                        case ';': 
                        case ' ': 
                        case '.': 
                        case '`': sc=2;break;
                        }
                }

                if(sc<2 && strlen(tz)<200-strlen(path)) {
                        strcat(path,tz);
                        if(stat(path,&sb)>=0 && !S_ISDIR(sb.st_mode)) {
                                f=fopen(tzfile,"w");
                                if(f!=NULL) {
                                        int pid;
                                        char *save=getenv("CONFIG_SAVE");
                                        fprintf(f,"%s\n",tz);
                                        fclose(f);
                                        HTML_OK;
                                        HTML_TITLE("Setting the Timezone","Setting the Timezone");
                                        printf("<center><p align=center>Your new timezone is <b>%s</b>!</p>\n",tz);

                                        printf("save=%s\n",save);
                                        //if(!(pid=fork())) { // we just assume it works
                                                //close(1);
                                                //close(2);
                                                system("freeze");
                                        //} else wait(NULL);

                                        unlink("/tmp/localtime");
                                        sprintf(path,"/usr/share/zoneinfo/%s",tz);
                                        symlink(path,"/tmp/localtime");

                                        gettimeofday(&now,NULL);
                                        tm=localtime(&now.tv_sec);
                                        printf("<center><p align=center>current time: %s<p align=center>\n",asctime(tm));
                                        printf("Note: most programs will pick up the new timezone automatically,<br>but you may want to reboot the system anyway.\n");
                                        printf("</p></center>");
                                        HTML_FOOTER;
                                        exit(0);
                                } else HTML_ERROR(TIMEZONE " file cannot be written");
                        }
                } 
                HTML_ERROR("Illegal Parameter");
        } else {
                // create selection table
                struct dirent **list;
                char path[200];
                int n,i;
                int found=0;
                
                n=scandir(ZONEDIR,&list,selectname,alphasort);
                if(n<=0) HTML_ERROR(ZONEDIR " not found");
                HTML_OK;
                HTML_TITLE("Timezone","Timezone");
                if(strlen(current)) printf("<h3 align=center>Curret timezone: <b>%s</b></h3>\n",current);

                gettimeofday(&now,NULL);
                tm=localtime(&now.tv_sec);
                printf("<p align=center><center>current time: %s</center><p>\n",asctime(tm));

                printf("<table align=center border=0>");

                // show current timezone at the very beginning
                for(found=i=0;i<n && !found && current[0];i++) {
                        struct stat sb;
                        strcpy(path,ZONEDIR "/");
                        strncat(path,list[i]->d_name,150);
                        if(strlen(list[i]->d_name)>145) continue; // long names are unacceptable
                        if(stat(path,&sb)>=0 &&  S_ISDIR(sb.st_mode)) {
                                struct dirent **sublist;
                                int sn,si;
                                char subpath[300];
                                sn=scandir(path,&sublist,selectname,alphasort);
                                for(si=0;si<sn && !found;si++) {
                                        strcpy(subpath,list[i]->d_name);
                                        strcat(subpath,"/");
                                        strncat(subpath,sublist[si]->d_name,100);
                                        if(!strcmp(subpath,current)) {
                                                printf("<tr><form action=%s method=get><td><select name=TZ>\n",getenv("SCRIPT_NAME"));
                                                for(si=0;si<sn && !found;si++) {
                                                        strcpy(subpath,list[i]->d_name);
                                                        strcat(subpath,"/");
                                                        strncat(subpath,sublist[si]->d_name,100);
                                                        printf("<option value=\"%s\" %s>%s</option>\n",subpath,strcmp(subpath,current)?"":"SELECTED",subpath);
                                                }
                                                printf("</select></td><td><input type=submit name=BUTTON value=\"Set Timezone\"></td></form></tr>\n");
                                                found=1;
                                                break;
                                        }
                                }
                                //printf("</select></td><td><input type=submit value=\"Set Timezone\"></td></form></tr>\n");
                        }
                }

                for(i=0;i<n;i++) {
                        struct stat sb;
                        strcpy(path,ZONEDIR "/");
                        strncat(path,list[i]->d_name,150);
                        if(strlen(list[i]->d_name)>145) continue; // long names are unacceptable
                        if(stat(path,&sb)>=0 &&  S_ISDIR(sb.st_mode)) {
                                struct dirent **sublist;
                                int sn,si;
                                char subpath[300];
                                printf("<tr><form action=%s method=get><td><select name=TZ>\n",getenv("SCRIPT_NAME"));
                                sn=scandir(path,&sublist,selectname,alphasort);
                                for(si=0;si<sn;si++) {
                                        strcpy(subpath,list[i]->d_name);
                                        strcat(subpath,"/");
                                        strncat(subpath,sublist[si]->d_name,100);
                                        printf("<option value=\"%s\" %s>%s</option>\n",subpath,strcmp(subpath,current)?"":"SELECTED",subpath);
                                }
                                printf("</select></td><td><input name=BUTTON type=submit value=\"Set Timezone\"></td></form></tr>\n");
                        }
                }

                printf("<tr><form action=%s method=get><td><select name=TZ>\n",getenv("SCRIPT_NAME"));
                for(i=0;i<n;i++) {
                        struct stat sb;
                        strcpy(path,ZONEDIR "/");
                        strncat(path,list[i]->d_name,150);
                        if(strlen(list[i]->d_name)>145) continue; // long names are unacceptable
                        if(stat(path,&sb)>=0 && !S_ISDIR(sb.st_mode))
                                printf("<option value=\"%s\" %s>%s</option>\n",list[i]->d_name,strcmp(list[i]->d_name,current)?"":"SELECTED",list[i]->d_name);
                }
                printf("</select></td><td><input name=BUTTON type=submit value=\"Set Timezone\"></td></form></tr>\n");
                printf("</table></body></html>\n");
                HTML_FOOTER;
        }
}
