/************************************************************************
   spawn.c - gives an interactive root shell on specified tty device

   Copyright (C) 2000 Transmeta Corporation

   written by Nathan I. Laredo <nlaredo@transmeta.com>

   This program is modifiable/redistributable under the terms
   of the GNU General Public Licence.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *************************************************************************/

#include "busybox.h"
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>  
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h> 
#include <sys/wait.h>   
#include <stdlib.h>
#include <grp.h>

char *progname;			/* name of program */

#if defined BB_SU
const char spawn_usage[] =
        "spawn ttydevice [uid [gid [cmdline]]]\n"
#ifndef BB_FEATURE_TRIVIAL_HELP
        "\nExecute a command (default is a shell) on a given tty.\n"
#endif
        ;
#endif 

void error_die (const char *fmt, ...)
{
	va_list arg_ptr;
	int save = errno;

	fflush(0);
	va_start(arg_ptr, fmt);
	fprintf(stderr, "%s: ", progname);
	vfprintf(stderr, fmt, arg_ptr);
	fprintf(stderr, ": %s\n", strerror(save));
	va_end(arg_ptr);
	exit(1);
}

int spawn_main(int argc, char **argv)
{
	int fd, uid = 0, gid = 0,pid;
	struct stat st;
        struct termios p;

	if (argc < 2) {
		usage(spawn_usage);
	}

	progname = argv[0];

	if(stat(argv[1], &st) < 0) {
		error_die("stat of %s failed", argv[1]);
	}
	if (!S_ISCHR(st.st_mode)) {
		fprintf(stderr, "%s is not a character device\n", argv[1]);
		exit(1);
	}
	if (argc > 2) {
		uid = atoi(argv[2]);
	}
	if (argc > 3) {
		gid = atoi(argv[3]);
	}
	umask(0);
	close(0);
	close(1);
	close(2);
	if ((pid=fork())) {
		int status;
		do { waitpid(pid,&status,0); } while(kill(pid,0)==0);
		exit(0);
	}
	if (setsid() < 0) {
		error_die("setsid failed");
	}
	if (chdir("/") < 0) {
		error_die("chdir failed");
	}
	if((fd = open(argv[1], O_RDWR)) < 0) {
		error_die("open of %s failed", argv[1]);
	}
	if (ioctl(fd, TIOCSCTTY, 0) < 0) {
		error_die("ioctl failed");
	}
	dup(fd);
	dup(fd);
	if (ioctl(fd, TIOCSCTTY, 1) < 0) {
		error_die("ioctl failed");
	}

	setregid(gid, gid);
	setgroups(1, &gid);
	setreuid(uid, uid);

        tcgetattr(0,&p);
        p.c_lflag|= ISIG | ICANON | ECHO | ECHOE | ECHOK | ECHOCTL | ECHOKE | IEXTEN;             
        tcsetattr(0,TCSANOW,&p);

	/* one of these should work */
	if (argc > 4) {	/* user specified command line, with path search */
		execvp(argv[4], argv + 4);
	}
	execl("/bin/bash", "/bin/bash", NULL);
	execl("/bin/ksh", "/bin/ksh", NULL);
	execl("/bin/sh", "/bin/sh", NULL);
	execl("/bin/ash", "/bin/ash", NULL);
	exit(1);
}
