/*
 * Copyright (C) 2000  Transmeta Corporation
 *
 * written by Jeff Uphoff <juphoff@transmeta.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "busybox.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <net/route.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#define LOOPBACK_ADDR "127.0.0.1"
#define LOOPBACK_NET "127.0.0.0"
#define LOOPBACK_MASK "255.0.0.0"
#define LOOPBACK_DEVICE "lo"


#if defined BB_UPLO
const char uplo_usage[] =
        "uplo\n"
#ifndef BB_FEATURE_TRIVIAL_HELP
        "\nInitialize 127.0.0.1 interface (lo).\n"
#endif
        ;
#endif 

int
uplo_main (int argc,char *argv[])
{
  struct ifreq req;
  struct rtentry rt;
  int sock;
  unsigned long int loopback_addr, loopback_mask, loopback_net;

  if(argc!=1) usage(uplo_usage);
  if ((sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1) {
    perror ("uplo (socket)");
    return errno;
  }
  memset (&req, 0, sizeof (req));
  strcpy (req.ifr_name, LOOPBACK_DEVICE);
  req.ifr_addr.sa_family = AF_INET;
  loopback_addr = inet_addr (LOOPBACK_ADDR);
  memcpy (&req.ifr_addr.sa_data[2], &loopback_addr, sizeof (loopback_addr));

  if (ioctl (sock, SIOCSIFADDR, &req) != 0) {
    perror ("uplo (ioctl, SIOCSIFADDR)");
    return errno;
  }
  req.ifr_flags |= IFF_UP;

  if (ioctl (sock, SIOCSIFFLAGS, &req) != 0) {
    perror ("uplo (ioctl, SIOCSIFFLAGS)");
    return errno;
  }
  memset (&rt, 0, sizeof (rt));
  rt.rt_dev = (char *)malloc (sizeof (LOOPBACK_DEVICE));
  strcpy (rt.rt_dev, LOOPBACK_DEVICE);
  rt.rt_flags |= RTF_UP;
  rt.rt_dst.sa_family = AF_INET;
  loopback_net = inet_addr (LOOPBACK_NET);
  memcpy (&rt.rt_dst.sa_data[2], &loopback_net, sizeof (loopback_net));
  rt.rt_genmask.sa_family = AF_INET;
  loopback_mask = inet_addr (LOOPBACK_MASK);
  memcpy (&rt.rt_genmask.sa_data[2], &loopback_mask, sizeof (loopback_mask));

  if (ioctl (sock, SIOCADDRT, &rt) != 0) {
    perror ("uplo (ioctl, SIOCADDRT)");
    return errno;
  }
  return 0;
}
